#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/rcc.h>

// applicazione di esempio. controlla un bottone e 2 led
// un led (quello built-in) semplicemente fa on/off (più on che off)
//
// un altro led (porta A5) si accende quando il bottone (porta A1) è premuto
// per ragioni di timer e semplicità le accensioni non sono molto immediate.
// chiaramente la risposta sarebbero gli interrupt, ma per ora facciamo così.
// Passo successivo: ancora niente interrupt, ma arrivano i thread

static void gpio_setup(void) {

  // built-in LED
  rcc_periph_clock_enable(RCC_GPIOC);
  gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL,
                GPIO13);

  // ext. push button (yellow)
  rcc_periph_clock_enable(RCC_GPIOA);
  gpio_set_mode(GPIOA, GPIO_MODE_INPUT, GPIO_CNF_INPUT_PULL_UPDOWN, GPIO1);

  // ext. led (red)
  gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO5);
}

int main(void) {
  int i;

  gpio_setup();
  gpio_set(GPIOA, GPIO5); // off by default

  char ledRedState = 1;
  for (;;) {
    gpio_clear(GPIOC, GPIO13);     /* LED on */
    for (i = 0; i < 15000000; i++) /* Wait a bit. */
      __asm__("nop");

    int pressed = !gpio_get(GPIOA, GPIO1);
    if (pressed) {
      gpio_clear(GPIOA, GPIO5);
    } else {
      gpio_set(GPIOA, GPIO5);
    }
    gpio_set(GPIOC, GPIO13);      /* LED off */
    for (i = 0; i < 5000000; i++) /* Wait a bit. */
      __asm__("nop");
  }

  return 0;
}
