#include "FreeRTOS.h"
#include "task.h"
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/rcc.h>

#ifdef __GNUC__
#define UNUSED(x) UNUSED_##x __attribute__((__unused__))
#else
#define UNUSED(x) UNUSED_##x
#endif

//add stack-overflow hook
extern void vApplicationStackOverflowHook(xTaskHandle *pxTask, signed portCHAR *pcTaskName);

// our tasks
void task_buttonled(void *args);
void task_onboardled(void *args);

// applicazione di esempio. controlla un bottone e 2 led
// un led (quello built-in) semplicemente fa on/off (più on che off)
//
// un altro led (porta A5) si accende quando il bottone (porta A1) è premuto (ovvero collegato alla  VCC)
// viene usato un thread per controllare questi due oggetti, in modo
// indipendente dal primo ciclo di on/off
//
// Passo successivo: interrupt

static void gpio_setup(void)
{

	// built-in LED
	rcc_periph_clock_enable(RCC_GPIOC);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL,
	              GPIO13);

	// ext. push button (yellow)
	rcc_periph_clock_enable(RCC_GPIOA);
	gpio_set_mode(GPIOA, GPIO_MODE_INPUT, GPIO_CNF_INPUT_PULL_UPDOWN, GPIO1);

	// ext. led (red)
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO5);
}

int main(void)
{
	gpio_setup();
	gpio_clear(GPIOA, GPIO5);

	xTaskCreate(task_buttonled, "btn", 100, NULL, configMAX_PRIORITIES - 1,
	            NULL);
	xTaskCreate(task_onboardled, "board", 100, NULL, configMAX_PRIORITIES - 1,
	            NULL);
	vTaskStartScheduler();
	for (;;) {
	}

	return 0;
}

void task_onboardled(void *UNUSED(args))
{
	while (1) {
		gpio_clear(GPIOC, GPIO13); /* LED on */
		vTaskDelay(pdMS_TO_TICKS(20));

		gpio_set(GPIOC, GPIO13); /* LED off */
		vTaskDelay(pdMS_TO_TICKS(40));
	}
}

void task_buttonled(void *UNUSED(args))
{
	while (1) {
		int pressed = !gpio_get(GPIOA, GPIO1);
		if (pressed) {
			gpio_clear(GPIOA, GPIO5);
		} else {
			gpio_set(GPIOA, GPIO5);
		}
		vTaskDelay(pdMS_TO_TICKS(50));
	}
}

/* three "small flashes" (an S in Morse code) on onboard led
 *
 * questa funzione viene chiamata in caso di stackoverflow su uno dei task; e'
 * quindi utile metterci un "segnale" riconoscibile per debug. In questo caso 3
 * flash rapidi.
 *
 * siccome "tarare" la dimensione dello stack necessario e' difficile, con
 * questa funzione si puo' andare "a tentoni" e ridurre la dimensione finche'
 * non si ottengono crash
 *
 * in questa funzione, a quanto pare (ma non ho trovato doc) non si possono
 * usare cose come vTaskDelay. Siamo quindi privi di RTOS e per fare gli sleep
 * dobbiamo fare i for "all'antica"
 */

void vApplicationStackOverflowHook(xTaskHandle *UNUSED(pxTask),
        signed portCHAR *UNUSED(pcTaskName))
{
    int i;
    for (;;) {
        for (char c = 0; c < 3; c++) {
            gpio_clear(GPIOC, GPIO13); /* LED on */
            for (i = 0; i < 1000000;
                    i++)                       /* small delay */
                __asm__("nop");

            gpio_set(GPIOC, GPIO13);       /* LED off */
            for (i = 0; i < 3000000; i++) { /* a bit more delay */
                __asm__("nop");
            }
        }
        for (i = 0; i < 10000000; i++) { /* long delay */
            __asm__("nop");
        }
    }
}
